import org.openkinect.processing.*; //kinect library
import codeanticode.syphon.*; //syphon library
import oscP5.*; //osc library
import netP5.*; //network library

Kinect2 kinect2; //kinect class instance
SyphonServer server; //syphon class instance for depth
SyphonServer server2; //syphon class instance for RGB
OscP5 oscP5; //OSC class instance
NetAddress myRemoteLocation; //network instance
int thresholdH = 1200; // max distance (in mm)
int thresholdL = 0; // min distance (in mm)
int[] depth; //depth mesurement array
PImage display; //if you need to draw image in processing
PGraphics canvas; //virtual image for Depth
PGraphics canvas2; //virtual image for RGB

void settings() {
  size(512, 424, P3D); //kinect V2 depth definition
  PJOGL.profile=1;
}

void setup() {
  //enable Kinect2
  kinect2 = new Kinect2(this);
  kinect2.initDepth();
  kinect2.initVideo();
  kinect2.initRegistered();
  kinect2.initDevice();
  // Make a blank image
  display = createImage(kinect2.depthWidth, kinect2.depthHeight, RGB);
  canvas = createGraphics(512, 424, P3D);
  canvas2 = createGraphics(512, 424, P3D);
  //create the syphon servers
  server = new SyphonServer(this, "imageDepth");
  server2 = new SyphonServer(this, "imageRGB");
  // declare local port and remote IP and port
  oscP5 = new OscP5(this,12000);
  myRemoteLocation = new NetAddress("127.0.0.1",1234);
}

void draw() {
  depth = kinect2.getRawDepth(); //read kinect depth
  canvas.beginDraw();
  canvas.loadPixels();
  // draw the depth image in white between high and low limits
  for (int x = 0; x < kinect2.depthWidth; x++) {
    for (int y = 0; y < kinect2.depthHeight; y++) {
      int offset = x + y * kinect2.depthWidth;
      int rawDepth = depth[offset];
      int pix = x + y*display.width;
      if (rawDepth > thresholdL && rawDepth < thresholdH) {
        canvas.pixels[pix] = color(255, 255, 255, 255); //draw white inside limits
      } else {
        canvas.pixels[pix] = color(0, 0, 0, 0); //draw black outside limits
      }
    }
  }
  canvas.updatePixels();
  canvas.endDraw();
  server.sendImage(canvas); // send depth image via syphon
  canvas2.beginDraw();
  canvas2.image(kinect2.getRegisteredImage(), 0, 0);
  canvas2.endDraw();
  server2.sendImage(canvas2); //send RGB registered image via syphon
}

void oscEvent(OscMessage theOscMessage) {
  if(theOscMessage.checkAddrPattern("/thresholdH")==true) { //receive osc high value
    /* check if the typetag is the right one. */
    if(theOscMessage.checkTypetag("i")) {
      /* parse theOscMessage and extract the values from the osc message arguments. */
      thresholdH = theOscMessage.get(0).intValue();
    }
  }
  if(theOscMessage.checkAddrPattern("/thresholdL")==true) { //receive osc low value
    /* check if the typetag is the right one. */
    if(theOscMessage.checkTypetag("i")) {
      /* parse theOscMessage and extract the values from the osc message arguments. */
      thresholdL = theOscMessage.get(0).intValue();
    }
  }
}